import logging
from typing import Any, Mapping
from colors import color


default_config = {
    logging.DEBUG: {"fg": "grey", "style": "bold+italic"},
    logging.INFO: {"fg": "green", "style": "bold+italic"},
    logging.WARNING: {"fg": "yellow", "style": "bold+italic"},
    logging.ERROR: {"fg": "red", "style": "bold+italic"},
    logging.CRITICAL: {"fg": "red", "style": "bold+italic+underline"},
}

default_exception_config = default_config[logging.CRITICAL]
default_stack_config = default_config[logging.ERROR]


class Formatter(logging.Formatter):
    """This log formatter wraps a given formatter and adds color to the output.

    If you want to use different colors from the default ones, you can pass a mapping to the color_configs argument in the constructor. This mapping should be from log levels (ints) to keyword arguments for the ansicolors library's color function. You can find more information on this library here: https://github.com/jonathaneunice/colors.

    The keys that you can use for each log level are:
    - fg (string): the foreground color
    - bg (string): the background color
    - style (string): any number of styles separated by a "+"

    The available colors are "black", "red", "green", "yellow", "blue", "magenta", "cyan", "white".
    The available styles are "none", "bold", "faint", "italic", "underline", "blink", "blink2", "negative", "concealed", "crossed".
    """

    def __init__(
        self,
        inner_formatter: logging.Formatter,
        color_configs: Mapping[int, Mapping[str, Any]] = default_config,
        exception_config: Mapping[str, Any] = default_exception_config,
        stack_config: Mapping[str, Any] = default_stack_config,
    ):
        """
        Args:
            inner_formatter: The formatter to use for the log messages.
            color_configs: A mapping from log levels to keyword arguments for the ansicolors library's color function.
            exception_config: The keyword arguments to pass to color for formatting exceptions.
            stack_config: The keyword arguments to pass to color for formatting stack traces.
        """
        self.formatter = inner_formatter
        self.configs = color_configs

    def format(self, record: logging.LogRecord) -> str:
        return color(
            self.formatter.format(record), **self.configs.get(record.levelno, {})
        )

    def formatException(exc_info):
        return color(self.formatter.formatException(exc_info), **self.exception_config)

    def formatStack(stack_info):
        return color(self.formatter.formatStack(stack_info), **self.stack_config)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rainbowlog']

package_data = \
{'': ['*']}

install_requires = \
['ansicolors>=1.1,<2.0', 'importlib-metadata>=4.11,<5.0']

setup_kwargs = {
    'name': 'rainbowlog',
    'version': '0.2.2',
    'description': 'Format your python logs with colours based on the log levels.',
    'long_description': "# Rainbow Log\n\nFormat your python logs with colours based on the log levels.\n\n## Installation\n\n\tpip install rainbowlog\n\n## Usage\n\nHere's a basic example of a script that logs colorfully to the console, but regularly to a file.\n\n```python\nimport logging\nimport rainbowlog\n\nlogger = logging.getLogger(__name__)\n\n# This one will write to the console\nstream_handler = logging.StreamHandler()\n\n# This one will write to a file\nfile_handler = logging.FileHandler('output.log')\n\n# Here we decide how we want the logs to look like\nformatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')\n\n# We want the stream handler to be colorful\nstream_handler.setFormatter(rainbowlog.Formatter(formatter))\n\n# We don't want the file handler to be colorful\nfile_handler.setFormatter(formatter)\n\n# Finally we add the handlers to the logger\nlogger.addHandler(stream_handler)\nlogger.addHandler(file_handler)\n\nif __name__ == '__main__':\n\tlogger.debug('This is a debug message')\n\tlogger.info('This is an info message')\n\tlogger.warning('This is a warning message')\n\tlogger.error('This is an error message')\n\tlogger.critical('This is a critical message')\n```\n\n## Docs\n\nYou can find the documentation [here](https://abrahammurciano.github.io/rainbowlog/rainbowlog)\n",
    'author': 'Abraham Murciano',
    'author_email': 'abrahammurciano@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/abrahammurciano/rainbowlog',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
